
import expect from 'expect'
import React, { Component } from 'react'
import ReactDOM from 'react-dom'
import { connect } from 'react-redux'

import Page from '../../src/page/Page'
import NavigationController from '../../src/navigation/NavigationController'

class testA extends Component{
    render(){ return (<div></div>) }
}

let PageAsynInvalid={
    getComponent(cb){
        cb(testA);
    }
};

class TestPage extends Page{
    willUnActive() {}
}

class PageIndex extends TestPage{
    render(){ return (<div></div>) }
}

class PageList extends TestPage{
    render(){ return (<div></div>) }
}

let PageListRouter={
    getComponent(cb){
        cb(PageList);
    }
};

let backStr,nativeBarClickStr;
class PageDetail extends TestPage{
    render(){ return (<div></div>) }
    pageBackReceiveResult(action,params,pageCode){
        let obj={action:action,params:params,pageCode:pageCode};
        backStr=JSON.stringify(obj);
    }
    executeNativeClick(uri){
        nativeBarClickStr=uri
    }
    saveInstanceState(){
        return 'pageDetail';
    }
}

class PageEditor extends TestPage{
    render(){ return (<div></div>) }
    saveInstanceState(){
        return 'pageEditor';
    }
}

class PageReduxInstance{
    getWrappedInstance(){
        return new PageDetail({pageId:112});
    }
}

class MultiPageRedux{
    getWrappedInstance(){
        return {
            getWrappedInstance(){
                return new PageDetail({pageId:112});
            }
        }
    }
}

class PageRedux extends TestPage{
    render(){ return (<div></div>) }
}
let PageReduxConnect=connect()(PageRedux);

let pages={
    '/pageIndex':PageIndex,
    '/pageList':PageListRouter,
    '/pageDetail': PageDetail,
    '/pageEditor': PageEditor,
    '/pageRedux': PageReduxConnect,
    '/pageInvalid': testA,
    '/pageAsynInvalid': PageAsynInvalid
};

let isRestore = false;
window.mutants.isRestore = () => { return isRestore; };


let container=window.document.createElement("div");
window.document.body.appendChild(container);
let navigationController=ReactDOM.render(<NavigationController pages={pages} rootPage={'/pageIndex'} />,container);


let box;
function create(){
    box=window.document.createElement("div");
    window.document.body.appendChild(box);
    return ReactDOM.render(<NavigationController pages={pages} rootPage={'/pageIndex'} />,box);
}

function destory(){
    ReactDOM.unmountComponentAtNode(box);
    window.document.body.removeChild(box);
}

function sleep(){
    let t=new Date().getTime(),mark=true;
    while(mark){
        if(new Date().getTime()-t>20) mark=false;
    }
}

function clearProp(data,name){
    let d=Object.assign([],data);
    d.forEach((item) => {
        delete item[name];
    });
    return d;
}

function clearStack(stackData){
    let stack = JSON.stringify(stackData);
    stack = JSON.parse(stack);
    stack = clearProp(stack, 'hash');
    stack = clearProp(stack, 'state');
    stack = clearProp(stack, 'pageId');
    return stack;
}

describe('NavigationController的堆栈测试', function () {

    describe('NavigationController实例化',function(){
        it('创建一个实例navigationController', function () {
            expect(navigationController).toExist();
        });
    });

    describe('API测试', function () {

        beforeEach(function(done){
            setTimeout(function(){
                done();
            },200);
        });

        describe('push 方法测试',function(){
            it('push 一个未注册的页面 /test', function () {
                expect(function(){
                    navigationController.push('/test');
                }).toThrow(/Page没有注册/);
            });
            it('push 一个已注册的页面 /pageDetail', function () {
                navigationController.push('/pageDetail');
                sleep();
                let pageStack=navigationController.state.pageStack,
                    pageJson={name:"/pageDetail",params:""};
                expect(pageStack).toExist("state.pageStack不存在");
                expect(pageStack.length).toBe(2,"state.pageStack长度不匹配，应为2");
                pageStack=clearStack(pageStack);
                expect(JSON.stringify(pageStack[1])).toBe(JSON.stringify(pageJson),"pageStack中page数据不对");
            });
            it('push 一个已注册的页面 /pageEditor 并带上参数pageParams,pageCode', function(){
                navigationController.push('/pageEditor',{param:1},'editor');
                sleep();
                let pageStack=navigationController.state.pageStack,
                    pageJson={name:"/pageEditor",params:{param:1},pageCode:'editor'};
                pageStack=clearStack(pageStack);
                expect(JSON.stringify(pageStack[2])).toBe(JSON.stringify(pageJson),"pageStack中page数据不对,检查params和pageCode");
            });
        });

        describe('pop 方法测试',function(){
            it('pop 出栈到当前页面', function () {
                let pageStack=navigationController.state.pageStack,
                    pageNum=pageStack.length,
                    currentPage=pageStack[pageNum-1].name;
                navigationController.pop();
                sleep();
                expect(pageStack.length).toBe(pageNum-1);
                expect(JSON.stringify(pageStack).indexOf('"name":"/'+currentPage+'",')).toBe(-1);
            });
        });

        describe('popAndReturn 方法测试',function(){
            it('popAndReturn 出栈并附带行为和参数,但行为的值不是ok/cancel',function(){
                let pageStack=navigationController.state.pageStack,
                    pageNum=pageStack.length,
                    pageStr,
                    backResult=JSON.stringify({action:"ok",params:{param:'back'},pageCode:"editor"});
                if(pageNum==1){
                    navigationController.push('/pageDetail',null,'editor');
                    sleep();
                }
                navigationController.push('/pageEditor',null,'editor');
                sleep();
                expect(function(){
                    navigationController.popAndReturn(123,{param:'back'});
                }).toThrow(/页面返回的action参数不正确/);
                navigationController.popToRoot();
                sleep();
            });
            it('popAndReturn 出栈并附带行为和参数',function(){
                let pageStack=navigationController.state.pageStack,
                    pageNum=pageStack.length,
                    pageStr,
                    backResult=JSON.stringify({action:"ok",params:{param:'back'},pageCode:"editor"});
                if(pageNum==1){
                    navigationController.push('/pageDetail',null,'editor');
                    sleep();
                }
                backStr=null;
                pageNum=pageStack.length;
                pageStr=JSON.stringify(pageStack);

                navigationController.push('/pageEditor',null,'editor');
                sleep();
                navigationController.popAndReturn("ok",{param:'back'});
                sleep();

                expect(pageStack.length).toBe(pageNum);
                expect(JSON.stringify(pageStack)).toBe(pageStr,"栈页面与期望不一致");
                expect(backStr).toBe(backResult);
            });
        });

        describe('popTo 方法测试',function(){
            it('popTo 出栈不带参数',function(){
                expect(function(){
                    navigationController.popTo();
                }).toThrow(/popTo参数不正确/);
            });
            it('popTo 出栈到一个不存在的栈页面',function(){
                navigationController.push('/pageDetail');
                navigationController.push('/pageEditor');
                expect(function(){
                    navigationController.popTo('/test');
                }).toThrow(/没有在当前栈找到相应的页面/);
            });
            it('popTo 出栈到一个存在的栈页面',function(){
                navigationController.popTo('/pageDetail');
                let pageStack=navigationController.state.pageStack;
                expect(pageStack[pageStack.length-1].name).toBe("/pageDetail");
            });
        });

        describe('popToRoot 方法测试',function(){
            it('popToRoot 出栈到根页面', function () {
                navigationController.push('/pageDetail');
                navigationController.popToRoot();
                let pageStack=navigationController.state.pageStack;
                expect(pageStack.length).toBe(1);
                expect(JSON.stringify(pageStack).indexOf('"name":"/pageIndex",')).toBeGreaterThan(-1);
            });
            it('popToRoot 当前已经是Root,不能再pop', function () {
                expect(function(){
                    navigationController.popToRoot();
                }).toThrow(/已经是页面栈中的唯一元素/);
            });
        });

        describe('replace 方法测试',function(){
            it('replace 非root节点', function () {
                let stack=navigationController.state.pageStack;
                if(stack.length==1){
                    navigationController.push('/pageDetail');
                    sleep();
                }
                let stack2=stack.concat();
                stack2.pop();
                stack2.push({name:"/pageEditor",params:""});
                let stackStr=JSON.stringify(clearStack(stack2));
                navigationController.replace('/pageEditor');
                sleep();
                stack=clearStack(stack);
                expect(JSON.stringify(stack)).toBe(stackStr);
            });
            it('replace root节点', function () {
                let stack=navigationController.state.pageStack;
                navigationController.popToRoot();
                sleep();
                navigationController.replace('/pageDetail',{test:1});
                sleep();
                expect(stack.length).toBe(1);
                stack=clearStack(stack);
                expect(JSON.stringify(stack)).toBe(JSON.stringify([{name:"/pageDetail",params:{test:1}}]));
            });
            it('replace 一个非page类的页面',function(){
                expect(function(){
                    navigationController.replace("/pageInvalid");
                }).toThrow(/不是一个可使用的Page对象/);
            });
        });

        describe('forkTo 方法测试',function(){
            it('在root节点上forkTo，直接push一个页面', function () {
                let stack=navigationController.state.pageStack;
                if(stack.length>1){
                    navigationController.popToRoot();
                    sleep();
                }
                let rootPage=JSON.stringify(clearStack(stack)[0]);
                navigationController.forkTo('/pageEditor',{forkTo:1});
                sleep();
                expect(stack.length).toBe(2);
                stack=clearStack(stack);
                expect(JSON.stringify(stack[0])).toBe(rootPage);
                expect(JSON.stringify(stack[1])).toBe(JSON.stringify({name:"/pageEditor",params:{forkTo:1}}));
            });
            it('在非root节点上forkTo,退回到root页面，再push一个页面', function () {
                let stack=navigationController.state.pageStack;
                if(stack.length==1){
                    navigationController.push('/pageDetail');
                    sleep();
                }
                let rootPage=JSON.stringify(clearStack(stack)[0]);
                navigationController.forkTo('/pageEditor',{forkTo:2});
                sleep();
                expect(stack.length).toBe(2);
                stack=clearStack(stack);
                expect(JSON.stringify(stack[0])).toBe(rootPage);
                expect(JSON.stringify(stack[1])).toBe(JSON.stringify({name:"/pageEditor",params:{forkTo:2}}));
            });
            it('在非root节点上forkTo到root节点,退回到root页面', function () {
                let stack=navigationController.state.pageStack;
                if(stack.length==1){
                    navigationController.push('/pageDetail');
                    sleep();
                }
                let rootPage=JSON.stringify(clearStack(stack)[0]);
                navigationController.forkTo(stack[0].name);
                sleep();
                stack=clearStack(stack);
                expect(stack.length).toBe(1);
                expect(JSON.stringify(stack[0])).toBe(rootPage);
            });
        });
    });

    describe('私有方法测试', function () {

        describe('_isPageClassLoaded Page类的加载',function(){

            let setState=navigationController.setState;
            let loadMap=navigationController.loadMap;

            beforeEach(function(){
                navigationController.loadMap={};
                navigationController.setState=function(){};
            });
            it('正常的Page页面，且页面不需要异步加载', function () {
                let nextState={
                    pageStack:[
                        {name:'/pageIndex',params:''},
                        {name:'/pageDetail',params:''}
                    ]
                };
                let result=navigationController._isPageClassLoaded(null,nextState);
                expect(result).toBe(true);
                expect(navigationController.loadMap["/pageIndex"]).toBe(PageIndex);
                expect(navigationController.loadMap["/pageDetail"]).toBe(PageDetail);
            });
            it('正常的Page页面，且需要异步加载', function () {
                let nextState={
                    pageStack:[
                        {name:'/pageList',params:''}
                    ]
                };
                let result=navigationController._isPageClassLoaded(null,nextState);
                expect(result).toBe(false);
            });
            it('恢复模式的stack，一个页面', function () {
                let nextState={
                    pageStack:[
                        {name:'/pageIndex',params:'',isRestored:true}
                    ]
                };
                let result=navigationController._isPageClassLoaded(null,nextState);
                expect(result).toBe(true);
                expect(navigationController.loadMap["/pageIndex"]).toBe(PageIndex);
            });
            it('恢复模式的stack，多个页面', function () {
                let nextState={
                    pageStack:[
                        {name:'/pageIndex',params:'',isRestored:true},
                        {name:'/pageDetail',params:'',isRestored:true}
                    ]
                };
                let result=navigationController._isPageClassLoaded(null,nextState);
                expect(result).toBe(true);
                expect(navigationController.loadMap["/pageIndex"]).toBe("waitPage");
                expect(navigationController.loadMap["/pageDetail"]).toBe(PageDetail);
            });
            it('waitPage状态的重新加载', function () {
                let nextState={
                    pageStack:[
                        {name:'/pageIndex',params:''}
                    ]
                };
                navigationController.loadMap["/pageIndex"]="waitPage";
                let result=navigationController._isPageClassLoaded(null,nextState);
                expect(result).toBe(true);
                expect(navigationController.loadMap["/pageIndex"]).toBe(PageIndex);
            });
            it('页面中有未注册的页面', function () {
                let nextState={
                    pageStack:[
                        {name:'/pageXXX',params:''}
                    ]
                };
                let result=navigationController._isPageClassLoaded(null,nextState);
                expect(result).toBe(true);
                expect(navigationController.loadMap["/pageXXX"]).toBe("unRegisterPage");
            });
            it('页面中有注册了，但是该页面为无效页面的', function () {
                let nextState={
                    pageStack:[
                        {name:'/pageInvalid',params:''}
                    ]
                };
                let result=navigationController._isPageClassLoaded(null,nextState);
                expect(result).toBe(true);
                expect(navigationController.loadMap["/pageInvalid"]).toBe("invalidPage");
            });

            afterEach(function(){
                navigationController.loadMap=loadMap;
                navigationController.setState=setState;
            });

        });

        describe('_loadPageClass Page类的加载',function(){

            beforeEach(function(){
                navigationController.loadMap={};
            });

            it('正常的Page页面，不需要异步加载',function(){
                let check="";
                let state=navigationController._loadPageClass("/pageDetail",function(){
                    check="abc";
                });
                sleep();
                expect(check).toBe("abc");
                expect(navigationController.loadMap["/pageDetail"]).toBe(PageDetail);
            });
            it('正常的Page页面，需要异步加载',function(){
                let check="";
                navigationController._loadPageClass("/pageList",function(){
                    check="abc";
                });
                sleep();
                expect(check).toBe("abc");
                expect(navigationController.loadMap["/pageList"]).toBe(PageList);
            });
            it('未注册的正常Page页面',function(){
                expect(function(){
                    navigationController._loadPageClass("/pageXXX");
                }).toThrow(/Page没有注册/);
            });
            it('注册的不正常Page页面',function(){
                expect(function(){
                    navigationController._loadPageClass("/pageInvalid");
                }).toThrow(/不是一个可使用的Page对象/);
            });
        });

        describe('_asynLoadPageClass',function(){

            let setState=navigationController.setState;
            let loadMap=navigationController.loadMap;
            let tmpState;

            beforeEach(function(){
                navigationController.loadMap={};
                navigationController.setState=function(state){
                    tmpState=state;
                };
            });

            it('加载正确的页面Page页面类',function(){
                let nextState={
                    pageStack:[
                        {name:'/pageList',params:''}
                    ]
                };
                navigationController._asynLoadPageClass({
                    pageName:'/pageList',
                    PageModules:pages['/pageList'],
                    nextState:nextState
                });
                expect(navigationController.loadMap["/pageList"]).toBe(PageList);
            });

            it('加载不正确的Page页面类',function(){
                let nextState={
                    pageStack:[
                        {name:'/pageAsynInvalid',params:''}
                    ]
                };
                navigationController._asynLoadPageClass({
                    pageName:'/pageAsynInvalid',
                    PageModules:pages['/pageAsynInvalid'],
                    nextState:nextState
                });
                expect(navigationController.loadMap["/pageAsynInvalid"]).toBe("invalidPage");
            });

            afterEach(function(){
                navigationController.loadMap=loadMap;
                navigationController.setState=setState;
            });
        });

        describe('_renderPage 输出一个Page，render中调用', function () {
            it('Page还没有加载完成', function () {
                expect(function(){
                    navigationController._renderPage({name:'/test',params:''},1);
                }).toThrow(/Page还没有加载完成/);
            });
            it('Page没有注册', function () {
                expect(function(){
                    navigationController.loadMap["/test"]="unRegisterPage";
                    navigationController._renderPage({name:'/test',params:''},1);
                }).toThrow(/Page没有注册/);
                delete navigationController.loadMap["/test"];
            });
            it('Page不是一个可使用的Page对象', function () {
                expect(function(){
                    navigationController.loadMap["/test"]="invalidPage";
                    navigationController._renderPage({name:'/test',params:''},1);
                }).toThrow(/不是一个可使用的Page对象/);
                delete navigationController.loadMap["/test"];
            });
            it('Page暂时不输出，主要用于刷新时，从localStorege中恢复栈的优化处理', function () {
                navigationController.loadMap["/test"]="waitPage";
                let page=navigationController._renderPage({name:'/test',params:''},1);
                expect(page.type).toBe('div');
                expect(JSON.stringify(page.props)).toBe('{}');
                delete navigationController.loadMap["/test"];
            });
            it('Page可用，但不是当前要显示的页面', function () {
                let page,
                    stack=navigationController.state.pageStack;
                if(stack.length==1){
                    navigationController.push("/pageDetail");
                    sleep();
                }
                page=navigationController._renderPage({name:stack[0].name,params:''},0);
                expect(page.type).toBe('div');
                expect(page.props.className.indexOf('navigationPage')).toBeGreaterThan(-1);
                expect(page.props.className.indexOf('currentPage')).toBe(-1);
            });
            it('Page可用，并且是当前要显示的页面', function () {
                let page,
                    stack=navigationController.state.pageStack;
                page=navigationController._renderPage({name:stack[stack.length-1].name,params:''},stack.length-1);
                expect(page.type).toBe('div');
                expect(page.props.className.indexOf('navigationPage')).toBeGreaterThan(-1);
                expect(page.props.className.indexOf('currentPage')).toBeGreaterThan(-1);
            });
            it('Page出现其他非正常值', function () {
                expect(function(){
                    navigationController.loadMap["/test"]="abcd";
                    navigationController._renderPage({name:'/test',params:''},1);
                }).toThrow(/这不是一个可使用的Page对象/);
                delete navigationController.loadMap["/test"];
            });
        });

        describe('_isPageObject 测试传入的对象是否一个基于Page继承的对象', function () {
            it('非page对象', function () {
                expect(navigationController._isPageObject(testA)).toBe(false);
            });
            it('直接继承的page对象', function () {
                expect(navigationController._isPageObject(PageIndex)).toBe(true);
            });
            it('被connect包装过的继承的page对象', function () {
                expect(navigationController._isPageObject(PageReduxConnect)).toBe(true);
            });
        });

        describe('_getPageInstance 返回传入对象的Page对象的实例',function(){
            it('直接继承的page对象',function(){
                let p1=new PageIndex({pageId:111}),
                    p2=navigationController._getPageInstance(p1);
                expect(p2.pageBackReceiveResult).toExist();
            });
            it('被connect包装过的继承的page对象', function () {
                let p1=new PageReduxInstance(),
                    p2=navigationController._getPageInstance(p1);
                expect(p2.pageBackReceiveResult).toExist();
            });
            it('被connect包装过n次的继承的page对象', function () {
                let p1=new MultiPageRedux(),
                    p2=navigationController._getPageInstance(p1);
                expect(p2.pageBackReceiveResult).toExist();
            });
        });

        describe('_popWithParams 将前一个页面返回的结果参数传递到当前页面',function(){
            it('没有参数需要传递', function () {
                navigationController.popParams=null;
            });
            it('有参数需要传递', function () {
                let popParams={action:'ok',params:{a:1},pageCode:'abc'};
                backStr=null;
                navigationController.popParams=popParams;
                navigationController._popWithParams(new PageDetail({pageId:113}));
                expect(backStr).toBe(JSON.stringify(popParams));
            });
            it('当前Page没有pageBackReceiveResult', function () {
                navigationController.popParams={action:'ok',params:{a:1},pageCode:'abc'};
                expect(function(){
                    navigationController._popWithParams(new testA());
                }).toThrow(/无法找到pageBackReceiveResult/);
            });
        });

        describe('_saveLocalStorage 将页面数据保存到LocalStorage', function () {
            let data = {
                pageStack:[{ a: 1 },{ b: 2 }]
            };
            it('测试调用_saveLocalStorage后，window.localStorage.pageStack值是否正确', function () {
                navigationController._saveLocalStorage(data);
                expect(window.localStorage.pageStack).toBe(JSON.stringify(data));
            });
        });

        describe('_restoreStackState 从LocalStorage中获取页面数据', function () {
            let data = {
                pageStack:[{ a: 1 },{ b: 2 }]
            };
            let outputdata={
                pageStack:[{a:1,isRestored:true},{b:2,isRestored:true}]
            };
            it('LocalStorage中有页面数据', function () {
                navigationController._saveLocalStorage(data);
                var output = navigationController._restoreStackState();
                expect(JSON.stringify(output)).toBe(JSON.stringify(outputdata));
            });
            it('LocalStorage没有页面数据', function () {
                window.localStorage.removeItem('pageStack');
                var output = navigationController._restoreStackState();
                expect(JSON.stringify(output)).toBe(JSON.stringify({pageStack:[]}));
            });
        });

        describe('_checkPageParams 检查传递的页面参数，防止function作为值传递',function(){
            it('参数为空,undefined,null',function(){
                let result;
                result=navigationController._checkPageParams();
                expect(result).toBe(true);
                result=navigationController._checkPageParams(undefined);
                expect(result).toBe(true);
                result=navigationController._checkPageParams(null);
                expect(result).toBe(true);
            });
            it('参数为值类型: 数字，字符串，布尔',function(){
                let result;
                result=navigationController._checkPageParams(123);
                expect(result).toBe(true);
                result=navigationController._checkPageParams("abc");
                expect(result).toBe(true);
                result=navigationController._checkPageParams(true);
                expect(result).toBe(true);
            });
            it('参数为function',function(){
                let result;
                result=navigationController._checkPageParams(function(){});
                expect(result).toBe(false);
            });
            it('参数为数组，数组为空',function(){
                let result,
                    data=[];
                result=navigationController._checkPageParams(data);
                expect(result).toBe(true);
            });
            it('参数为数组，数组不为空，每一项都是值类型，不含function',function(){
                let result,
                    data=[123,'abc',true,undefined,null];
                result=navigationController._checkPageParams(data);
                expect(result).toBe(true);
            });
            it('参数为数组，数组不为空，含一个function，其他项都是值类型',function(){
                let result,
                    data=[123,'abc',true,undefined,null,function(){}];
                result=navigationController._checkPageParams(data);
                expect(result).toBe(false);
            });
            it('参数为JsonObject，{}',function(){
                let result;
                result=navigationController._checkPageParams({});
                expect(result).toBe(true);
            });
            it('参数为JsonObject，为单层结构，每一个key的value都为值类型',function(){
                let result,
                    data={
                        a:123,
                        b:"abc",
                        c:true,
                        d:undefined,
                        e:null
                    };
                result=navigationController._checkPageParams(data);
                expect(result).toBe(true);
            });
            it('参数为JsonObject，为单层结构，有一个key的value是function,其余key的value都为值类型',function(){
                let result,
                    data={
                        a:123,
                        b:"abc",
                        c:true,
                        d:undefined,
                        e:null,
                        f:function(){}
                    };
                result=navigationController._checkPageParams(data);
                expect(result).toBe(false);
            });
            it('参数为复杂JsonObject，包含多层，有数组，不含function',function(){
                let result,
                    data={
                        a:123,
                        b:[123,'abc',true,undefined,null,{a:123,b:false,d:[]}]
                    };
                result=navigationController._checkPageParams(data);
                expect(result).toBe(true);
            });
            it('参数为复杂JsonObject，包含多层，有数组，最深层有function',function(){
                let result,
                    data={
                        a:123,
                        b:[123,'abc',true,undefined,null,{a:123,b:false,d:[],e:function(){}}]
                    };
                result=navigationController._checkPageParams(data);
                expect(result).toBe(false);
            });
            it('来自类的实例化对象',function(){
                let result,
                    data=new testA();
                result=navigationController._checkPageParams(data);
                expect(result).toBe(false);
            });
            it('参数为复杂JsonObject，包含多种值类型和引用类型，包含一个类的实例化对象',function(){
                let result,
                    data={
                        a:123,
                        b:[123,'abc',true,undefined,null,{a:123,b:false,d:[],e:new testA()}]
                    };
                result=navigationController._checkPageParams(data);
                expect(result).toBe(false);
            });
        });

        describe('_onExecutePageClick',function(){

            it('Native的NavigateBar的点击事件传递到对应的Page进行响应',function(){
                expect(window.executePageClick).toNotBe(undefined);
                navigationController.forkTo("/pageDetail");
                sleep();
                window.executePageClick("abc");
                expect(nativeBarClickStr).toBe("abc");
            });

        });

        //TODO:待完成
        describe('_onExcutePageStateStore',function(){

            it('Native调取应用现场保护的命令',function(){
                if(navigationController.state.pageStack.length>1){
                    navigationController.popToRoot();
                    sleep();
                }
                navigationController.replace("/pageDetail");
                sleep();
                navigationController.push("/pageEditor");
                sleep();
                expect(window.executePageStateStore).toNotBe(undefined);
                window.executePageStateStore();
                const pageState = window.localStorage.pageState;
                //const testState = JSON.stringify(['pageDetail','pageEditor']);
                //expect(pageState).toBe(testState);
            });

        });

        describe('_init 初始化处理',function(){

            beforeEach(function(){
                navigationController.setState({pageStack:[]});
            });

            it('没有hash的时候',function(){
                navigationController._initStack();
                let stack=navigationController.state.pageStack;
                expect(stack.length).toBe(1);
                stack=clearStack(stack);
                expect(JSON.stringify(stack[0])).toBe(JSON.stringify({name:"/pageIndex",params:""}));
            });

            it('有hash,且localStorage中有保存数据,但当前hash不指向栈的最后一个页面',function(){
                let mockStack={
                    pageStack:[
                        {name:'/pageIndex',params:""},
                        {name:'/pageDetail',params:""},
                        {name:'/pageEditor',params:""}
                    ]
                };
                let outputStack=[
                    {name:'/pageIndex',params:""}
                ];
                window.localStorage.pageStack=JSON.stringify(mockStack);
                navigationController._initStack("#/pageDetail/12");
                let stack=navigationController.state.pageStack;
                expect(stack.length).toBe(1);
                stack=clearStack(stack);
                expect(JSON.stringify(stack)).toBe(JSON.stringify(outputStack));
            });

            it('有hash,且localStorage中有保存数据,且当前hash指向栈的最后一个页面',function(){
                let mockStack={
                    pageStack:[
                        {name:'/pageIndex',params:""},
                        {name:'/pageDetail',params:""},
                        {name:'/pageEditor',params:""}
                    ]
                };
                let outputStack=[
                    {name:'/pageIndex',params:"",isRestored:true},
                    {name:'/pageDetail',params:"",isRestored:true},
                    {name:'/pageEditor',params:""}
                ];
                window.localStorage.pageStack=JSON.stringify(mockStack);
                navigationController._initStack("#/pageEditor/13");
                let stack=navigationController.state.pageStack;
                expect(stack.length).toBe(3);
                stack=clearStack(stack);
                expect(JSON.stringify(stack)).toBe(JSON.stringify(outputStack));
            });

            it('有hash，但是浏览器不支持localStorage或者localStorag没有数据',function(){
                window.localStorage.removeItem("pageStack");
                navigationController._initStack("#/pageEditor/14");
                let stack=navigationController.state.pageStack;
                expect(stack.length).toBe(1);
                stack=clearStack(stack);
                expect(JSON.stringify(stack[0])).toBe(JSON.stringify({name:"/pageIndex",params:""}));
            });

        });

    });
});