/**
 * Created by guopeng on 16/3/30.
 */

import expect from 'expect'
import React from 'react';
import { mount, shallow } from 'enzyme';
import immutable from 'immutable'

import LinkButton from '../../src/page/LinkButton'
import Menu from '../../src/page/Menu'

describe('menu', () => {

    function setup(props = {}){
        const wrapper = shallow(<Menu {...props} />);
        return wrapper;
    }

    var opts =  {
        scope:{navigationController:{}},
        item:[]
    };

    function cloneOpts(){
        return immutable.Map({}).mergeDeep(opts).toJS();
    }

    
    it('render, menu应该是由一个button和一个弹出列表组成', () => {
        var menu = setup(cloneOpts());
        expect(menu.children().length).toBe(2);
        expect(menu.children().first().type()).toBe(LinkButton);
        expect(menu.children().last().type()).toBe('ul');
    } )
    
    it('render, 根据传入的item配置,渲染出正确的列表结构', () => {
        var opt = cloneOpts();
        opt.item = [{text:'tx1',onClick:()=>{}},{text:'tx2',onClick:()=>{}}];
        var menu = setup(opt);
        expect(menu.find('ul').children().length).toBe(2);
        expect(menu.find('ul').children().first().text()).toBe('tx1')
    })
    
    it('配置hide是true时,整个menu应该不显示', () => {
        var opt = cloneOpts();
        opt.hide = true;
        var menu = setup(opt);
        expect(menu.prop('style').display).toBe('none');
    })
    
    it('点击按钮时, pop列表处于隐藏状态时应该显示', () => {
        var opt = cloneOpts();
        opt.item = [{text:'tx1',onClick:()=>{}},{text:'tx2',onClick:()=>{}}];
        var menu = setup(opt);
        menu.children().first().simulate('click');
        expect(menu.find('ul').prop('style').display).toBe('block');
    })

    it('点击按钮时, pop列表处于显示状态时应该隐藏', () => {
        var opt = cloneOpts();
        opt.item = [{text:'tx1',onClick:()=>{}},{text:'tx2',onClick:()=>{}}];
        var menu = setup(opt);
        menu.children().first().simulate('click');
        menu.children().first().simulate('click');
        expect(menu.find('ul').prop('style').display).toBe('none');
    })

    it('点击page内容容器时,pop处于显示状态时应该隐藏, _contentClick', () => {
        var opt = cloneOpts();
        opt.item = [{text:'tx1',onClick:()=>{}},{text:'tx2',onClick:()=>{}}];
        var menu = setup(opt);
        menu.children().first().simulate('click');
        menu.instance()._contentClick();
        expect(menu.instance().state.ulStyle.display).toBe('none');
    })

    it('点击菜单项时,对应的事件因该被调用', () => {
        var opt = cloneOpts();
        opt.item = [{text:'tx1',onClick:expect.createSpy()},{text:'tx2',onClick:()=>{}}];
        var menu = setup(opt);
        menu.find('ul').children().first().simulate('click');

        expect(opt.item[0].onClick).toHaveBeenCalled();
    })

})
